@@ -81,7 +81,7 @@
 
 package com.lowagie.text.pdf.codec;
 
-import com.lowagie.text.pdf.ICC_Profile;
+import java.awt.color.ICC_Profile;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.DataInputStream;
@@ -93,7 +93,7 @@ import java.util.zip.InflaterInputStream;
 
 import com.lowagie.text.ExceptionConverter;
 import com.lowagie.text.Image;
-//SID import com.lowagie.text.ImgRaw;
+import com.lowagie.text.ImgRaw;
 import com.lowagie.text.pdf.ByteBuffer;
 import com.lowagie.text.pdf.PdfArray;
 import com.lowagie.text.pdf.PdfDictionary;
@@ -421,7 +421,7 @@ public class PngImage {
                 byte iccp[] = PdfReader.FlateDecode(icccom, true);
                 icccom = null;
                 try {
-                    icc_profile = ICC_Profile.getInstance(iccp, (colorType & 2) == 0 ? 1 : 3);
+                    icc_profile = ICC_Profile.getInstance(iccp);
                 }
                 catch (Exception e) {
                     icc_profile = null;
